# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.jsonable_encoder import jsonable_encoder
from ...types.async_passthrough_reciept import AsyncPassthroughReciept
from ...types.data_passthrough_request import DataPassthroughRequest
from ...types.remote_response import RemoteResponse

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AsyncPassthroughClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(self, *, request: DataPassthroughRequest) -> AsyncPassthroughReciept:
        """
        Asynchronously pull data from an endpoint not currently supported by Merge.

        Parameters:
            - request: DataPassthroughRequest.
        ---
        from merge import (DataPassthroughRequest, MethodEnum,
                           MultipartFormFieldRequest, RequestFormatEnum)
        from merge.client import Merge

        client = Merge(account_token="YOUR_ACCOUNT_TOKEN", api_key="YOUR_API_KEY", )
        client.filestorage.create(request=DataPassthroughRequest(method=MethodEnum.GET, path="/scooters", data="{"company": "Lime", "model": "Gen 2.5"}", multipart_form_data=[MultipartFormFieldRequest(name="resume", data="SW50ZWdyYXRlIGZhc3QKSW50ZWdyYXRlIG9uY2U=", file_name="resume.pdf", content_type="application/pdf", )], request_format=RequestFormatEnum.JSON, ), )
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/filestorage/v1/async-passthrough"),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(AsyncPassthroughReciept, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def retrieve(self, async_passthrough_receipt_id: str) -> RemoteResponse:
        """
        Retrieves data from earlier async-passthrough POST request

        Parameters:
            - async_passthrough_receipt_id: str.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/filestorage/v1/async-passthrough/{async_passthrough_receipt_id}",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(RemoteResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncAsyncPassthroughClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(self, *, request: DataPassthroughRequest) -> AsyncPassthroughReciept:
        """
        Asynchronously pull data from an endpoint not currently supported by Merge.

        Parameters:
            - request: DataPassthroughRequest.
        ---
        from merge import (DataPassthroughRequest, MethodEnum,
                           MultipartFormFieldRequest, RequestFormatEnum)
        from merge.client import AsyncMerge

        client = AsyncMerge(account_token="YOUR_ACCOUNT_TOKEN", api_key="YOUR_API_KEY", )
        await client.filestorage.create(request=DataPassthroughRequest(method=MethodEnum.GET, path="/scooters", data="{"company": "Lime", "model": "Gen 2.5"}", multipart_form_data=[MultipartFormFieldRequest(name="resume", data="SW50ZWdyYXRlIGZhc3QKSW50ZWdyYXRlIG9uY2U=", file_name="resume.pdf", content_type="application/pdf", )], request_format=RequestFormatEnum.JSON, ), )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/filestorage/v1/async-passthrough"),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(AsyncPassthroughReciept, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def retrieve(self, async_passthrough_receipt_id: str) -> RemoteResponse:
        """
        Retrieves data from earlier async-passthrough POST request

        Parameters:
            - async_passthrough_receipt_id: str.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/filestorage/v1/async-passthrough/{async_passthrough_receipt_id}",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(RemoteResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)
